@@ -1,6 +1,7 @@
 use crate::error::Error;
 use crate::math_helper;
 use crate::math_helper::float_eq;
+use crate::vector::Vector;
 
 use super::vector2::Vector2;
 use super::vector3::Vector3;
@@ -21,6 +22,124 @@ pub struct Matrix2x2 {
     pub r2: Vector2,
 }
 
+pub trait Matrix {
+    #[allow(non_snake_case)]
+    /// Identity Matrrix NxN, where all elements Uij that i = j are `1`
+    fn IDENTITY() -> Self;
+    ///Matrix NxN determinant
+    fn det(&self) -> f32;
+    ///Transforms a MatrixNxN into a Vec<Vec<f32>> by row
+    fn vectorize(&self) -> Vec<Vec<f32>>;
+    fn modulus(&self) -> f32;
+    /// Transpose of Matrix NxN.
+    /// Transpose switches rows and columns
+    fn transpose(&self) -> Self;
+    /// Inverse of a Matrix NxN
+    fn inverse(self, delta: f32) -> Result<Self, Error>
+    where
+        Self: Sized;
+}
+
+impl Matrix for Matrix3x3 {
+    /// | 1 0 0 |
+    /// | 0 1 0 |
+    /// | 0 0 1 |
+    fn IDENTITY() -> Self {
+        Matrix3x3::new_idx(1f32, 0f32, 0f32, 0f32, 1f32, 0f32, 0f32, 0f32, 1f32)
+    }
+
+    /// `               | 4 -3 0 |`
+    /// `let matrix =   | 2 -1 2 |`
+    /// `               | 1  5 7 |`
+    /// `matrix.det() = -32`
+    ///
+    /// ```
+    /// use mathf::matrix::{Matrix, Matrix3x3};
+    ///
+    /// let matrix = Matrix3x3::new_idx(4.0, -3.0, 0.0, 2.0, -1.0, 2.0, 1.0,  5.0, 7.0);
+    /// assert_eq!(matrix.det(), -32f32);
+    /// ```
+    fn det(&self) -> f32 {
+        (self.r1.x * self.r2.y * self.r3.z
+            + self.r1.y * self.r2.z * self.r3.x
+            + self.r1.z * self.r2.x * self.r3.y)
+            - (self.r1.z * self.r2.y * self.r3.x
+                + self.r1.y * self.r2.x * self.r3.z
+                + self.r1.x * self.r2.z * self.r3.y)
+    }
+
+    /// ```
+    /// use mathf::matrix::{Matrix, Matrix3x3};
+    ///
+    /// let matrix = Matrix3x3::new_idx(4.0, -3.0, 0.0, 2.0, -1.0, 2.0, 1.0,  5.0, 7.0);
+    /// assert_eq!(matrix.vectorize(), vec![
+    ///    vec![4.0, -3.0, 0.0], vec![2.0, -1.0, 2.0], vec![1.0,  5.0, 7.0]
+    /// ]);
+    /// ```
+    fn vectorize(&self) -> Vec<Vec<f32>> {
+        vec![
+            self.r1.to_vector(),
+            self.r2.to_vector(),
+            self.r3.to_vector(),
+        ]
+    }
+
+    /// Modulus of a matrix 3x3
+    ///                | 4 -3 0 |
+    /// let matrix =   | 2 -1 2 |
+    ///                | 1  5 7 |
+    /// matrix.modulus() =
+    /// -> 4 | -1 2 | - (-3) | 2 2 | + | 2 -1 |
+    ///      |  5 7 |        | 1 7 |   | 1  5 |
+    fn modulus(&self) -> f32 {
+        let elements: Vec<f32> = (0..=2)
+            .map(|i| {
+                let idxs: Vec<usize> = (0..=2).filter(|x| x != &i).collect();
+                let factor = self.r1[i];
+                let inner2x2 = Matrix2x2::new_idx(
+                    self.r2[idxs[0]],
+                    self.r2[idxs[1]],
+                    self.r3[idxs[0]],
+                    self.r3[idxs[1]],
+                );
+                factor * inner2x2.det()
+            })
+            .collect();
+        elements[0] - elements[1] + elements[2]
+    }
+
+    fn transpose(&self) -> Matrix3x3 {
+        Matrix3x3::new_idx(
+            self.r1[0], self.r2[0], self.r3[0], self.r1[1], self.r2[1], self.r3[1], self.r1[2],
+            self.r2[2], self.r3[2],
+        )
+    }
+
+    /// `               | 1 2 3 |`
+    /// `let matrix =   | 0 1 5 |`
+    /// `               | 5  6 0 |`
+    /// `                           | -30  18  7 |`
+    /// `let matrix_inverse =  1/5  |  25 -15 -5 |`
+    /// `                           | -5   4   1 |`
+    ///
+    /// ```
+    /// use mathf::matrix::{Matrix, Matrix3x3};
+    ///
+    /// let matrix = Matrix3x3::new_idx(1.0, 2.0, 3.0, 0.0, 1.0, 5.0, 5.0,  6.0, 0.0);
+    /// let matrix_inverse = Matrix3x3::new_idx(-6.0, 3.6, 1.4, 5.0, -3.0, -1.0, -1.0,  0.8, 0.2);
+    /// assert_eq!(matrix.inverse(0.001).unwrap(), matrix_inverse);
+    /// ```
+    fn inverse(self, delta: f32) -> Result<Matrix3x3, Error> {
+        let modulus = self.modulus();
+        if float_eq(modulus, 0f32, delta) {
+            return Err(Error::SingularMatrixNotInversible);
+        }
+        let cofactor_matrix = self.cofactor();
+        let traspose = cofactor_matrix.transpose();
+        Ok(traspose / modulus)
+    }
+}
+
 impl Matrix3x3 {
     ///Creates a new Matrix3x3 from 3 vector3 rows
     pub fn new(r1: Vector3, r2: Vector3, r3: Vector3) -> Matrix3x3 {
@@ -50,48 +169,7 @@ impl Matrix3x3 {
         }
     }
 
-    #[allow(dead_code, non_snake_case)]
-    pub fn IDENTITY() -> Matrix3x3 {
-        Matrix3x3::new_idx(1f32, 0f32, 0f32, 0f32, 1f32, 0f32, 0f32, 0f32, 1f32)
-    }
-
-    ///Matrix 3x3 determinant
-    pub fn det(&self) -> f32 {
-        (self.r1.x * self.r2.y * self.r3.z
-            + self.r1.y * self.r2.z * self.r3.x
-            + self.r1.z * self.r2.x * self.r3.y)
-            - (self.r1.z * self.r2.y * self.r3.x
-                + self.r1.y * self.r2.x * self.r3.z
-                + self.r1.x * self.r2.z * self.r3.y)
-    }
-
-    ///Transforms a Matrix3x3 into a Vec<Vec<f32>>
-    pub fn vectorize(&self) -> Vec<Vec<f32>> {
-        vec![
-            self.r1.to_vector(),
-            self.r2.to_vector(),
-            self.r3.to_vector(),
-        ]
-    }
-
-    /// Modulus of a matrix 3x3
-    pub fn modulus(&self) -> f32 {
-        let elements: Vec<f32> = (0..=2)
-            .map(|i| {
-                let idxs: Vec<usize> = (0..=2).filter(|x| x != &i).collect();
-                let factor = self.r1[i];
-                let inner2x2 = Matrix2x2::new_idx(
-                    self.r2[idxs[0]],
-                    self.r2[idxs[1]],
-                    self.r3[idxs[0]],
-                    self.r3[idxs[1]],
-                );
-                factor * inner2x2.det()
-            })
-            .collect();
-        elements[0] - elements[1] + elements[2]
-    }
-
+    /// [wiki](https://en.wikipedia.org/wiki/Minor_(linear_algebra))
     pub fn cofactor(&self) -> Matrix3x3 {
         let elements1: Vec<f32> = (0..=2)
             .map(|i| {
@@ -143,24 +221,80 @@ impl Matrix3x3 {
             elements3[2],
         )
     }
+}
 
-    /// Transpose of Matrix 3x3
-    pub fn transpose(&self) -> Matrix3x3 {
-        Matrix3x3::new_idx(
-            self.r1[0], self.r2[0], self.r3[0], self.r1[1], self.r2[1], self.r3[1], self.r1[2],
-            self.r2[2], self.r3[2],
-        )
+impl Matrix for Matrix2x2 {
+    /// | 1 0 |
+    /// | 0 1 |
+    fn IDENTITY() -> Self {
+        Matrix2x2::new_idx(1f32, 0f32, 0f32, 1f32)
     }
 
-    /// Inverse of a Matrix 3x3
-    pub fn inverse(self, delta: f32) -> Result<Matrix3x3, Error> {
-        let modulus = self.modulus();
-        if float_eq(modulus, 0f32, delta) {
-            return Err(Error::SingularMatrixNotInversible);
+    /// `               | 4 -3 |`
+    /// `let matrix =   | 0  2 |`
+
+    /// `matrix.det() = 2`
+    ///
+    /// ```
+    /// use mathf::matrix::{Matrix, Matrix2x2};
+    ///
+    /// let matrix = Matrix2x2::new_idx(4.0, -3.0, 0.0, 2.0);
+    /// assert_eq!(matrix.det(), 8f32);
+    /// ```
+    fn det(&self) -> f32 {
+        self.r1.x * self.r2.y - self.r1.y * self.r2.x
+    }
+
+    /// `               | 4 -3 |`
+    /// `let matrix =   | 0  2 |`
+
+    /// `matrix.det() = 2`
+    ///
+    /// ```
+    /// use mathf::matrix::{Matrix, Matrix2x2};
+    ///
+    /// let matrix = Matrix2x2::new_idx(4.0, -3.0, 0.0, 2.0);
+    /// assert_eq!(matrix.vectorize(), vec![
+    ///    vec![4.0, -3.0], vec![0.0, 2.0]    
+    /// ]);
+    /// ```
+    fn vectorize(&self) -> Vec<Vec<f32>> {
+        vec![self.r1.to_vector(), self.r2.to_vector()]
+    }
+
+    /// Same as determinant for Matrix 2x2
+    fn modulus(&self) -> f32 {
+        (self.r1.x * self.r2.y) - (self.r2.x * self.r1.y)
+    }
+
+    fn transpose(&self) -> Self {
+        Matrix2x2::new_idx(self.r1.x, self.r2.x, self.r1.y, self.r2.y)
+    }
+
+    /// `               | 4 7 |`
+    /// `let matrix =   | 2 6 |`
+    ///
+    /// `                            |  6 -7 |`
+    /// `let matrix_inverse =  1/10  | -2  4 |`
+    ///
+    /// ```
+    /// use mathf::matrix::{Matrix, Matrix2x2};
+    ///
+    /// let matrix = Matrix2x2::new_idx(4.0, 7.0, 2.0, 6.0);
+    /// let matrix_inverse = Matrix2x2::new_idx(0.6, -0.7, -0.2, 0.4);
+    /// assert_eq!(matrix.inverse(0.001).unwrap(), matrix_inverse);
+    /// ```
+    fn inverse(self, delta: f32) -> Result<Matrix2x2, Error> {
+        let det = self.clone().det();
+
+        if math_helper::float_eq(det, 0f32, delta) {
+            Err(Error::NonZeroDeterminantMatrix)
+        } else {
+            Ok(Matrix2x2::new(
+                Vector2::new(self.r2.y / det, -self.r1.y / det),
+                Vector2::new(-self.r2.x / det, self.r1.x / det),
+            ))
         }
-        let cofactor_matrix = self.cofactor();
-        let traspose = cofactor_matrix.transpose();
-        Ok(traspose / modulus)
     }
 }
 
@@ -177,30 +311,6 @@ impl Matrix2x2 {
             r2: Vector2::new(n3, n4),
         }
     }
-
-    #[allow(dead_code, non_snake_case)]
-    pub fn IDENTITY() -> Matrix2x2 {
-        Matrix2x2::new_idx(1f32, 0f32, 0f32, 1f32)
-    }
-
-    ///Matrix 2x2 determinant
-    pub fn det(self) -> f32 {
-        self.r1.x * self.r2.y - self.r1.y * self.r2.x
-    }
-
-    ///Inverse of a Matrix 2x2
-    pub fn inverse(self, delta: f32) -> Result<Matrix2x2, Error> {
-        let det = self.clone().det();
-
-        if math_helper::float_eq(det, 0f32, delta) {
-            Err(Error::NonZeroDeterminantMatrix)
-        } else {
-            Ok(Matrix2x2::new(
-                Vector2::new(self.r2.y / det, -self.r1.y / det),
-                Vector2::new(-self.r2.x / det, self.r1.x / det),
-            ))
-        }
-    }
 }
 
 impl ops::Add for Matrix3x3 {
@@ -492,6 +602,20 @@ mod tests_matrix2x2 {
         let expected = Matrix2x2::new_idx(-2f32, 1f32, 1.5f32, -0.5f32);
         assert_eq!(expected, matrix.inverse(0.001f32).unwrap());
     }
+
+    #[test]
+    fn modulus() {
+        let matrix = Matrix2x2::new_idx(1f32, 2f32, 3f32, 4f32);
+        let expected = -2f32;
+        assert_eq!(expected, matrix.modulus());
+    }
+
+    #[test]
+    fn transpose() {
+        let matrix = Matrix2x2::new_idx(1f32, 2f32, 3f32, 4f32);
+        let expected = Matrix2x2::new_idx(1f32, 3f32, 2f32, 4f32);
+        assert_eq!(expected, matrix.transpose());
+    }
 }
 
 #[cfg(test)]
